/***********************************************************
 *          JNI interface for DAMOTE                       *
 *          version 1.0.2704                               *
 *          All parameters configurable!                   *
 ***********************************************************/

#include "be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE.h"
#include <jni.h>
#include "database/database_api.h"
#include "database/database_util.h"
#include "computation/computation_st.h"
#include "computation/computation_api.h"
#include "computation/backup/backup_api.h"
#include "common/common.h"
#include "common/setup.h"
#include "error/error.h"

#include <stdio.h>
#include <string.h>

#define MAX_SESSIONS 128

DataBase* centralDB;
int NBOA;
int NBPL;

typedef struct globalVars {
	DataBase * centralDB;
	DAMOTEConfig damoteConfig;
	ErrorList errorManager;
} SessionItem;

typedef SessionItem* SessionItemPtr;

SessionItemPtr sessions[MAX_SESSIONS];

//session id of current context (only useful for debugging purposes)
int currentSessId = -1;

//last created sessionId
int lastSession = -1;

//total number of sessions
int nbSessions = -1;

int init() {
	nbSessions = 0;
	memset(sessions, 0, MAX_SESSIONS*sizeof(SessionItemPtr));
	//fprintf(stderr, "Init sessions array\n");
	return 0;	
}

int saveCntx(int sessId) {
    sessions[sessId]->centralDB = centralDB;
    sessions[sessId]->damoteConfig = damoteConfig;
    sessions[sessId]->errorManager = errorManager;
    return 0;
}

int loadCntx(JNIEnv *env, jobject obj) {
    jclass objclass = (*env)->GetObjectClass(env, obj);
    jfieldID fid = (*env)->GetFieldID(env, objclass, "sessId", "I");
    int sessId = (*env)->GetIntField(env, obj, fid);

	if (sessions[sessId] == 0) {
    	addError(CRITICAL, "Non existing Session (%d) in %s at line %d", sessId,
    		__FILE__,__LINE__);
		return -1;
	}

	centralDB = sessions[sessId]->centralDB;
	errorManager = sessions[sessId]->errorManager;
	damoteConfig = sessions[sessId]->damoteConfig;

	//debug
	//if (currentSessId != sessId) {
	//	fprintf(stderr, "Context changed for SessionId=%d\n", sessId);
	//}

	currentSessId = sessId;
	return sessId;
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniprintDamoteDB
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniprintDamoteDB(JNIEnv *env, jobject obj) {
	loadCntx(env, obj);
	DBprintDB(centralDB);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniinitDamote
 * Signature: (IIZZZFFFFFFF[[F)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniinitDamote(JNIEnv *env, jobject obj, jint nbOA, jint nbPL, jboolean colorClause, jboolean allowReroute, jboolean capacityClause, jfloat loadbal, jfloat load, jfloat sqLoad, jfloat relativeLoad, jfloat sqRelativeLoad, jfloat delay, jfloat fortz, jobjectArray reroutingScorecoef){

    //Parameters not used: nbOA, nbPL don't configure DAMOTE
    //which works by default with 8 preemption levels!
    //and 8 category of services!
    int j,l;
    int k;

    // Init the error manager ....
    errorInit();

    if (nbSessions >= MAX_SESSIONS) {
    	addError(CRITICAL, "Too many Sessions (%d) in %s at line %d", nbSessions,
    		__FILE__,__LINE__);
    	return;
    }

    //initialise the structures (only useful at first run)
    if (nbSessions == -1) init();

    COST=0;

    NBOA = nbOA;
    NBPL = nbPL;

    // Allocate a new DB
    if ((centralDB = DBnew(0)) == NULL)
    {
	addError(CRITICAL,"Impossible to init a DataBase in %s at line %d",
		 __FILE__,__LINE__);
	return;
    }

    // Configuration
    damoteConfig.predicateConfig.colorClause=colorClause;
    damoteConfig.predicateConfig.allowReroute=allowReroute;
    damoteConfig.predicateConfig.capacityClause=capacityClause;
    memset(&damoteConfig.primaryComputationConfig,0,sizeof(PrimaryComputationConfig));


    // should be set for all OAs ! (TODO)
    damoteConfig.primaryComputationConfig.load[0]=load;
    damoteConfig.primaryComputationConfig.loadBal[0]=loadbal;
    damoteConfig.primaryComputationConfig.sqLoad[0]=sqLoad;
    damoteConfig.primaryComputationConfig.relLoad[0]=relativeLoad;
    damoteConfig.primaryComputationConfig.sqRelLoad[0]=sqRelativeLoad;
    damoteConfig.primaryComputationConfig.delay[0]=delay;
     damoteConfig.primaryComputationConfig.Fortz=fortz;



    memset(&damoteConfig.reroutingConfig,0,sizeof(ReroutingConfig));
    for (j=0;j<NBOA;j++){
	jfloatArray scoreArray = (*env)->GetObjectArrayElement(env,reroutingScorecoef,j);
	jfloat *body = (*env)->GetFloatArrayElements(env, scoreArray, 0);

	for (l=0;l<NBPL;l++){

	    damoteConfig.reroutingConfig.scoreCoef[j][l]=body[l];
	}

	(*env)->ReleaseFloatArrayElements(env,scoreArray,body,0);
	(*env)->DeleteLocalRef(env,scoreArray);
    }


    /* loop is finite because we checked number of sessions (nbSessions < MAX_SESSIONS) */
    k = (lastSession+1) % MAX_SESSIONS;
    while (sessions[k] != 0) {
    	k = (k+1) % MAX_SESSIONS;
    }
    lastSession = k;
    ++nbSessions;

    sessions[lastSession] = (SessionItemPtr)malloc(sizeof(SessionItem));
    saveCntx(lastSession);

    jclass objclass = (*env)->GetObjectClass(env, obj);
    jfieldID fid = (*env)->GetFieldID(env, objclass, "sessId", "I");
    (*env)->SetIntField(env, obj, fid, (jint)lastSession);

    currentSessId = lastSession;
    //fprintf(stderr, "New Session created with SessId: %d\n", lastSession);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jnikillDamote
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jnikillDamote(JNIEnv *env, jobject obj) {
	int sessId = loadCntx(env, obj);
    // Destroy the DB if required
    if (centralDB != NULL){
	DBdestroy(centralDB);
    }
    else {
	fprintf(stderr,"ERROR, call to killDamote before initDamote\n");
	return;
    }

    // Print list of error
    printErrorStack();

    // Destroy error manager
    errorDestroy();

    free(sessions[sessId]);
    sessions[sessId] = NULL;
    --nbSessions;
    currentSessId = -1;
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniaddNode
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniaddNode(JNIEnv *env, jobject obj, jint nodeId) {
    jclass newExcCls;
	int sessId = loadCntx(env, obj);
    //fprintf(stderr,"Adding node %d\n", nodeId);
    if (DBaddNode(centralDB, nodeId) != 0)
    {
	addError(CRITICAL,"Impossible to init a Node in %s at line %d",
		 __FILE__,__LINE__);

	saveCntx(sessId);

	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;
    }
    saveCntx(sessId);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniaddLink
 * Signature: (IIII[F[[F[[F[[F[[F)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniaddLink(JNIEnv *env, jobject obj, jint linkId, jint color, jint src, jint dst, jfloatArray cap, jobjectArray rbw, jobjectArray pbw, jobjectArray bbw, jobjectArray fbw){

    // parameters not used: color, pbw, bbw, fbw
    int j,k;
    DBLinkState *ls = NULL;
    jfloat *body;
    jfloatArray rbwArray;
    jfloat *body2;
    jclass newExcCls;
	int sessId = loadCntx(env, obj);
    //fprintf(stderr,"Call to addlink for src %d to dst %d\n", src,dst);

    if ((ls = DBlinkStateNew()) == NULL)
    {
	addError(CRITICAL,"Impossible to init a Link in %s at line %d",
		 __FILE__,__LINE__);
	saveCntx(sessId);
	return;
    }

    body = (*env)->GetFloatArrayElements(env, cap, 0);
    for (j=0;j<NBOA;j++){
	ls->cap[j] = (double) body[j];
        rbwArray = (*env)->GetObjectArrayElement(env,rbw,j);
	body2 = (*env)->GetFloatArrayElements(env, rbwArray, 0);
	for (k=0;k<NBPL;k++){
	    ls->rbw[j][k] = (double) body2[k];
	}
	(*env)->ReleaseFloatArrayElements(env,rbwArray,body2,0);
	(*env)->DeleteLocalRef(env,rbwArray);
    }
    (*env)->ReleaseFloatArrayElements(env, cap, body, 0);

    if (DBaddLink(centralDB, linkId, src, dst, ls) != 0)
    {
	addError(CRITICAL,"Impossible to add Link in %s at line %d",
		 __FILE__,__LINE__);

	saveCntx(sessId);
	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;
    }
    //fprintf(stderr,"Adding link %d src %d dest %d cap %f rbw %f NBOA %d NBPL %d\n", linkId, src, dst, ls->cap[0], ls->rbw[0][0], NBOA, NBPL);


    // Clear the ls
    DBlinkStateDestroy(ls);
    saveCntx(sessId);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniaddLSP
 * Signature: (I[IIIF)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniaddLSP(JNIEnv *env, jobject obj, jint lspid, jintArray path, jint PL, jint OA, jfloat bw){
    DBLabelSwitchedPath* lsp = NULL;
    LongList *preemptList;
    int u;
    jint* body;
    jsize len;
    jclass newExcCls;
    int sessId = loadCntx(env, obj);

     damoteConfig.predicateConfig.allowReroute=FALSE; // no preemption

    /*setmainframe();
    segv_setup();
    */
    lsp = DBlspNew();
    lsp->id = lspid;

    preemptList = longListNew(-1);

    //bandwidth affected to OA 0.
    for (u=0;u<NBOA;u++){
	if (u==OA) lsp->bw[u]=(double)bw;
	else lsp->bw[u]=0;
    }

    lsp->precedence = PL;



    body = (*env)->GetIntArrayElements(env, path, 0);
    len = (*env)->GetArrayLength(env, path);

    for (u=0;u<len;u++){
	longListPushBack(&(lsp->path),body[u]);
    };



    /*for (u=0;u<lsp->path.top;u++){
	fprintf(stderr," %d", (lsp->path).cont[u]);
	}*/


    lsp->type = PRIM;

    if (DBaddLSP(centralDB, lsp, preemptList) < 0)
    {
	addError(CRITICAL,"Impossible to add PRIMARY LSP (computed by another algorithm) in %s at line %d",
		 __FILE__,__LINE__);
	addError(CRITICAL, "Lsp ID: %ld, Request: %f\n", lspid, (double)bw);
	//fprintf(stderr,"Can't add primary LSP! \n");

	saveCntx(sessId);

	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;

    }
    saveCntx(sessId);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniaddBackupLSP
 * Signature: (II[IIIFZ)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniaddBackupLSP
(JNIEnv * env, jobject obj, jint lspid, jint plspid, jintArray path, jint PL, jint OA, jfloat bw, jboolean type) {
    DBLabelSwitchedPath* lsp = NULL;
    DBLabelSwitchedPath* primLsp = NULL;
    LongList *preemptList;
    int u;
    jint* body;
    jsize len;
    jclass newExcCls;
    int sessId = loadCntx(env, obj);

     damoteConfig.predicateConfig.allowReroute=FALSE; // no preemption

    lsp = DBlspNew();
    lsp->id = lspid;

    preemptList = longListNew(-1);

    //bandwidth affected to OA 0.
    for (u=0;u<NBOA;u++){
	if (u==OA) lsp->bw[u]=(double)bw;
	else lsp->bw[u]=0;
    }

    lsp->precedence = PL;

    body = (*env)->GetIntArrayElements(env, path, 0);
    len = (*env)->GetArrayLength(env, path);

    for (u=0;u<len;u++){
	longListPushBack(&(lsp->path),body[u]);
    };

    lsp->type = type == JNI_FALSE ? GLOBAL_BACK : LOCAL_BACK;

    lsp->primID = plspid;

    if ((primLsp = DBgetLSP(centralDB, plspid)) == NULL) {
	addError(CRITICAL,"Impossible to find PRIMARY LSP in %s at line %d",
		 __FILE__,__LINE__);
	addError(CRITICAL, "Lsp ID: %ld, Request: %f\n", lspid, (double)bw);

	saveCntx(sessId);

	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;
    }

    longListCopy(&(lsp->primPath), &(primLsp->path));


    if (DBaddLSP(centralDB, lsp, preemptList) < 0)
    {
	addError(CRITICAL,"Impossible to add BACKUP LSP (computed by another algorithm) in %s at line %d",
		 __FILE__,__LINE__);
	addError(CRITICAL, "Lsp ID: %ld, Request: %f\n", lspid, (double)bw);

	saveCntx(sessId);

	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;

    }
    saveCntx(sessId);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniremoveLSP
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniremoveLSP
(JNIEnv * env, jobject obj, jint lspid) {
	int sessId = loadCntx(env, obj);
jclass newExcCls;
//fprintf(stderr,"Call to remove lsp %d\n", lspid);
    if (DBremoveLSP(centralDB,lspid) < 0){
	addError(CRITICAL,"Impossible to remove LSP in %s at line %d",
		 __FILE__,__LINE__);
	//fprintf(stderr,"Can't remove primary LSP! \n");

	saveCntx(sessId);
	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;
    }
    saveCntx(sessId);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniremoveNode
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniremoveNode
(JNIEnv *env, jobject obj, jint nodeid) {
	int sessId = loadCntx(env, obj);
jclass newExcCls;
//fprintf(stderr,"Call to remove node %d\n", nodeid);
    if (DBremoveNode(centralDB,nodeid) < 0){
	addError(CRITICAL,"Impossible to remove Node in %s at line %d",
		 __FILE__,__LINE__);

	saveCntx(sessId);
	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;
    }
    saveCntx(sessId);

}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jniremoveLink
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jniremoveLink
  (JNIEnv *env, jobject obj, jint srcNodeId, jint dstNodeId){
jclass newExcCls;
	int sessId = loadCntx(env, obj);
    // fprintf(stderr,"Call to removelink for src %d to dst %d\n", srcNodeId,dstNodeId);

    if (DBremoveLink(centralDB,srcNodeId,dstNodeId) < 0){
	addError(CRITICAL,"Impossible to remove Link in %s at line %d",
		 __FILE__,__LINE__);

	saveCntx(sessId);
	newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
	if (newExcCls == NULL){
	    fprintf(stderr,"Unable to find the exception class, giving up\n");
	    return;
	}
	(*env)->ThrowNew(env,newExcCls,NULL);
	return;
    }
    saveCntx(sessId);
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jnicomputePath
 * Signature: (IIIIIIIIF[IZZ)[I
 */
JNIEXPORT jintArray JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jnicomputePath
  (JNIEnv *env, jobject obj, jint id, jint src, jint dst, jint rrid, jint rrsrc, jint rrdst, jint OA, jint PL, jfloat reservation, jintArray colorArray, jboolean ADDLSP, jboolean PREEMPT){

    // parameters not used: rrid, rrsrc, rrdst, colorArray
    int j;
    int u;
    int temp;
    jclass newExcCls;
    jintArray intarray;
    jintArray lspsarray;
    jfieldID fid2;
    jclass objclass;

    LSPRequest req;
    DBLabelSwitchedPath* lsp = NULL;
    LongList *preemptList;


	int sessId = loadCntx(env, obj);

/*    int path_length = 0;
      int lsps_length = 0;*/



    // first, we adapt the allow reroute parameter
    damoteConfig.predicateConfig.allowReroute=PREEMPT;



    lspRequestInit(&req);
    req.type=PRIM;
    req.precedence=PL; //add this arg

    req.id=id; // add this arg
    for (u=0;u<NBOA;u++){
	if (u==OA) {
	    req.bw[u]=(double)reservation;

	}
	else req.bw[u]=0;
    }



    //DBprintDB(centralDB);
    longListPushBack(&req.path,src);
    longListPushBack(&req.path,-1);
    longListPushBack(&req.path,dst);


    if (computePrimaryPath(centralDB,&req) < 0){
		addError(CRITICAL,"Impossible to compute Path in %s at line %d",
			 __FILE__,__LINE__);
		saveCntx(sessId);
		newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/NoRouteToHostException");
		if (newExcCls == NULL){
		    fprintf(stderr,"Unable to find the exception class, giving up\n");
		    return;
		}
		(*env)->ThrowNew(env,newExcCls,NULL);
		return;
    }



    // DBprintDB(centralDB);
    //gettimeofday(&time,&none);

    preemptList = longListNew(-1);
    lsp = DBlspNew();
    lsp->id = req.id;
    for (u=0;u<NBOA;u++){
		lsp->bw[u]=req.bw[u];
    }
    lsp->precedence = req.precedence;
    longListCopy(&(lsp->path),&(req.path));
    lsp->type = PRIM;

    if ((ADDLSP == 0) && (PREEMPT ==1)){
		if (getpreemptList(centralDB, lsp, preemptList) < 0)
		{
		    addError(CRITICAL,"Impossible to obtain preemptList in %s at line %d",
			     __FILE__,__LINE__);
		    //fprintf(stderr,"Can't obtain preemptList \n");
            printErrorStack();
			saveCntx(sessId);
		    newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/RoutingException");
		    if (newExcCls == NULL){
				fprintf(stderr,"Unable to find the exception class, giving up\n");
				return;
		    }
		    (*env)->ThrowNew(env,newExcCls,NULL);
		    return;
		}
    }
    else if (ADDLSP==1){
		if ((DBaddLSP(centralDB, lsp, preemptList)) < 0)
		{
		    addError(CRITICAL,"Impossible to add PRIMARY LSP in %s at line %d",
			     __FILE__,__LINE__);
			saveCntx(sessId);
		    newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
		    if (newExcCls == NULL){
			fprintf(stderr,"Unable to find the exception class, giving up\n");
			return;
		    }
		    (*env)->ThrowNew(env,newExcCls,NULL);
		    return;

		}
    }

    intarray = (*env)->NewIntArray(env,req.path.top);

    lspsarray = (*env)->NewIntArray(env,(*preemptList).top);

    objclass = (*env)->GetObjectClass(env, obj);


    //jfieldID fid = (*env)->GetStaticFieldID(env, class, "time", "I");

    //(*env)->SetStaticIntField(env, class, fid, timevalue);

    fid2 = (*env)->GetFieldID(env, objclass, "lsps", "[I");

    for (j=0;j<(*preemptList).top;j++)
    {
		temp = (int)(*preemptList).cont[j];

		// we directly remove all lsps that have to be preempted in DAMOTE too

		if (DBremoveLSP(centralDB,temp) < 0){
			addError(CRITICAL,"Impossible to remove LSP in %s at line %d",
				 __FILE__,__LINE__);
			//fprintf(stderr,"Can't remove primary LSP! \n");
			saveCntx(sessId);

			newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
			if (newExcCls == NULL){
			    fprintf(stderr,"Unable to find the exception class, giving up\n");
			    return;
			}
			(*env)->ThrowNew(env,newExcCls,NULL);
			return;
		}

		(*env)->SetIntArrayRegion(env,lspsarray,j,1,&temp);
    }

    if ((*preemptList).top != 0)
        (*env)->SetObjectField(env, obj, fid2, lspsarray);

    for (j=0;j<req.path.top;j++)
    {
		temp = (int)req.path.cont[j];
		(*env)->SetIntArrayRegion(env,intarray,j,1,&temp);
    }

    //fprintf(stderr,"Here's cost of computed path for lsp %d : %f\n", id, COST);
	saveCntx(sessId);
    return intarray;
}

/*
 * Class:     be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE
 * Method:    jnicomputeBackupPath
 * Signature: (I[IIZZ)[Ljava/lang/Object;
 */
JNIEXPORT jobjectArray JNICALL Java_be_ac_ulg_montefiore_run_totem_repository_DAMOTE_JNIDAMOTE_jnicomputeBackupPath
  (JNIEnv *env, jobject obj, jint primarylspId, jintArray backuplspIds, jint backupType, jboolean ADDLSP, jboolean PREEMPT){
	int sessId = loadCntx(env, obj);
    int j,u,l=0,temp;
    DBLabelSwitchedPath lsp;
    DBLabelSwitchedPath *templsp = NULL;
    LongVec rerouting;
    jclass tempClass;
    jintArray intarray;
    LSPRequestList req;
    jint* body;
    jclass newExcCls;
    jobjectArray lspsarray;
    DBLinkState* new = NULL;


    lspRequestListInit(&req, -1);
    intarray= (*env)->NewIntArray(env,100);
    tempClass = (*env)->GetObjectClass(env,intarray);

    // first, we adapt the allow reroute parameter
    damoteConfig.predicateConfig.allowReroute=PREEMPT;

    body = (*env)->GetIntArrayElements(env, backuplspIds, 0);




    if (backupType == 1){

	if (computeBackup(centralDB, &req, primarylspId, GLOBAL)<0){
	    addError(CRITICAL,"Impossible to compute global backup paths in %s at line %d",
		 __FILE__,__LINE__);
		saveCntx(sessId);
	    newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/NoRouteToHostException");
	    if (newExcCls == NULL){
		fprintf(stderr,"Unable to find the exception class, giving up\n");
		return;
	    }
	    (*env)->ThrowNew(env,newExcCls,NULL);
	    return;
	}
    }
    else{
	if (computeBackup(centralDB, &req, primarylspId, LOCAL)<0){
	    addError(CRITICAL,"Impossible to compute local backup paths in %s at line %d",
		 __FILE__,__LINE__);
		saveCntx(sessId);
	    newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/NoRouteToHostException");
	    if (newExcCls == NULL){
		fprintf(stderr,"Unable to find the exception class, giving up\n");
		return;
	    }
	    (*env)->ThrowNew(env,newExcCls,NULL);
	    return;
	}

    }


    lspsarray = (*env)->NewObjectArray(env,req.size,tempClass,NULL);

    DBlspInit(&lsp);

    // make the lsp

    // trying to find the path corresponding to the primary lsp
    // directly from the database (Olivier)

    if ((templsp = DBlspVecGet(&(centralDB->lspVec),primarylspId)) == NULL){
	addError(CRITICAL, "Not able to find primary LSP (id = %ld) in %s at line %d",
                 primarylspId,__FILE__,__LINE__);
	saveCntx(sessId);
        return;
    }

    longVecCopy(&(lsp.primPath), &(templsp->path));



    //fprintf(stderr,"\n");
   for (j=0;j<lsp.primPath.top;j++){
//	fprintf(stderr," %d", lsp.primPath.cont[j]);
       l++;
	}
   //fprintf(stderr,"\n");
   //fprintf(stderr,"Valeur de req.size %d\n", req.size);
   if (l-1!=req.size && backupType == 0){
       fprintf(stderr,"Pas assez de lsps de backups locaux pour la taille du lsp!\n");
   }

   for (j=0; j<req.size; j++) {

	intarray = (*env)->NewIntArray(env,req.cont[j].path.top);
	//fprintf(stderr,"Valeur de req.cont[j].path.top %d\n", req.cont[j].path.top);

	for (u=0;u<req.cont[j].path.top;u++)
	{
	    temp = (int)req.cont[j].path.cont[u];

	    (*env)->SetIntArrayRegion(env,intarray,u,1,&temp);
	}

	(*env)->SetObjectArrayElement(env,lspsarray,j,intarray);


	longVecCopy(&(lsp.path), &(req.cont[j].path));

	lsp.id = body[j];

	for (u=0;u<NBOA;u++){
	    lsp.bw[u]=req.cont[j].bw[u];

      	}

	lsp.precedence = 0;

	if (backupType == 1){
	    lsp.type = GLOBAL_BACK;
	}
	else lsp.type = LOCAL_BACK;

	lsp.primID = primarylspId;
	lsp.noContentionId = -1;

	//printf("Add LSP with id = %ld, primID = %ld\n", lsp.id, primarylspId);

	longVecInit(&rerouting,-1); // nothing is done with preemptionList, we don't even know how preemptions work when backup are used

	if ((ADDLSP == 0) && (PREEMPT ==1)){
	    if (getpreemptList(centralDB, &lsp, &rerouting) < 0)
	    {
		addError(CRITICAL,"Impossible to obtain preemptList in %s at line %d",
			 __FILE__,__LINE__);
		fprintf(stderr,"Can't obtain preemptList \n");
	    }
	}
	else if (ADDLSP==1){


	    if (DBaddLSP(centralDB, &lsp, &rerouting) < 0)
	    {
		addError(CRITICAL,"Impossible to add BACKUP LSP in %s at line %d",
			 			 __FILE__,__LINE__);
		fprintf(stderr,"Backup global non ajout pour primaire %d, valeur de la bande passante: %f: \n", primarylspId, lsp.bw[0]);
		for (u=0;u<(req.cont[j].path.top)-1;u++)
		{

		    //fprintf(stderr," %d", temp);


		    new = DBgetLinkState(centralDB,(int)req.cont[j].path.cont[u],(int)req.cont[j].path.cont[u+1]);

		    fprintf(stderr,"Valeur de la capacit du lien %d - %d: %f, Valeur de la bande passante rserve P0: %f, PetB0: %f\n", (int)req.cont[j].path.cont[u],(int)req.cont[j].path.cont[u+1], new->cap[0], new->pbw[0][0], new->rbw[0][0]);

		}
		fprintf(stderr," \n");

		//print the path of the backup


		//fprintf(stderr,"Can't add backup LSP! \n");
		saveCntx(sessId);
		newExcCls = (*env)->FindClass(env,"be/ac/ulg/montefiore/run/totem/repository/model/exception/AddDBException");
		if (newExcCls == NULL){
		    fprintf(stderr,"Unable to find the exception class, giving up\n");
		    return;
		}
		(*env)->ThrowNew(env,newExcCls,NULL);
		return;
	    }

	}

	longVecEnd(&rerouting);
   }

   lspRequestListEnd(&req);

   //DBprintDB(centralDB);
   //return;

	saveCntx(sessId);


return lspsarray;

   /* for (j=0;j<req.path.top;j++)
      {
      temp = (int)req.path.cont[j];

       (*env)->SetIntArrayRegion(env,intarray,j,1,&temp);
       }
   */


}


