#include "computation/backup/dijkstra.h"

CPTreeNode* CPnewTN()
{
    CPTreeNode* tn=NULL;

    if ((tn = calloc(1, sizeof(CPTreeNode))) == NULL)
    {
	addError(CRITICAL,"Impossible to allocated a new PrioQueue in %s at line %d",
		 __FILE__,__LINE__);
	return NULL;
    }

    return tn;
}

int CPdestroyTN(CPTreeNode* tn)
{
    if (tn == NULL)
    {
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    free(tn);

    return 0;
}


int CPinitPQ(CPPrioQueue* pq)
{
    if (pq == NULL)
    {
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    pq->root = NULL;
    pq->top = NULL;
    pq->size = 0;
    
    return 0;
}

CPPrioQueue* CPnewPQ()
{
    CPPrioQueue* pq=NULL;

    if ((pq = calloc(1, sizeof(CPPrioQueue))) == NULL)
    {
	addError(CRITICAL,"Impossible to allocated a new PrioQueue in %s at line %d",
		 __FILE__,__LINE__);
	return NULL;
    }

    /* Done by calloc !!!
    pq->root = NULL;
    pq->top = NULL;
    pq->size = 0;
    */

    return pq;
}

int CPendPQ(CPPrioQueue* pq)
{
    if (pq == NULL)
    {
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    while ((CPpopTop(pq)) != NULL);

    pq->size = 0;
    pq->root = NULL;
    pq->top = NULL;
	
    return 0;

}

int CPdestroyPQ(CPPrioQueue* pq)
{
    if (pq == NULL)
    {
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    if (CPendPQ(pq) < -1)
    {
	addError(CRITICAL,"Destruction incomplete in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    free(pq);

    return 0;

}

int CPinsertPQ(CPPrioQueue* pq,CPDijkNode* dn, double key)
{
    CPTreeNode* tn, *ptr, *lastPtr=NULL;
 
    if (pq == NULL)
    {
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }
   
    if ((tn = CPnewTN()) == NULL)
    {
	addError(CRITICAL,"Impossible to allocated a new TreeNode in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    tn->key = key;
    tn->node = dn;

    // size == 0
    if (pq->root == NULL)
    {
	pq->root = tn;
	pq->top = tn;
	pq->size++;
	return 0;
    }
    
    // lower than top 
    if (key <= pq->top->key)
    {
	pq->top->leq = tn;
	tn->father = pq->top;
	pq->top = tn;
	pq->size++;
	return 0;
    }

    // anywhere else ...
    ptr = pq->root;

    while (ptr != NULL)
    {
	lastPtr = ptr;

	if (key <= ptr->key)
	    ptr = ptr->leq;
	else
	    ptr = ptr->gt;
    }

    if (key <= lastPtr->key)
	lastPtr->leq = tn;
    else 
	lastPtr->gt = tn;
    
    tn->father = lastPtr;
    pq->size++;

    return 0;
}

CPDijkNode* CPpopTop(CPPrioQueue* pq)
{
    CPTreeNode* tn;
    CPDijkNode* dn;

    if (pq == NULL)
    {
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return NULL;
    }

    tn = pq->top;

    if (tn != NULL)
    {
	pq->size--;

	if (tn == pq->root)
	{
	    pq->root = tn->gt;
	    pq->top = tn->gt;
	    if (tn->gt != NULL)
		tn->gt->father = NULL;
	}
	else
	{
	    tn->father->leq = tn->gt;
	    if (tn->gt != NULL)
	    {
		pq->top = tn->gt;
		tn->gt->father = tn->father;
	    }
	    else
		pq->top = tn->father;
	}

	// now find the new top;
	if (pq->size > 0)
	    while (pq->top->leq != NULL)
		pq->top = pq->top->leq;	
    }
    else 
    {
	return NULL;
    }
    
    dn = tn->node;
    
    if (CPdestroyTN(tn) < 0)
    {
	addError(WARNING,"Unable to destroy TreeNode but DijkNode was returned in %s at line %d",
		 __FILE__,__LINE__);
    }

    return dn;
}
