#ifndef PRIMARYPATH_UTIL_H
#define PRIMARYPATH_UTIL_H

#include "computation/computation_api.h"

// Topology Structures
//---------------------

typedef struct BKConnectInfo_
{
    double gain[NB_OA];
} BKConnectInfo;

typedef struct BKConnect_
{
    long neighbId;
    DBLinkState *linkState; // pointeur directement sur la DB!
    BKConnectInfo info;
} BKConnect;

typedef struct BKConnectVec_
{
    long size;
    long top;
    BKConnect* cont;
} BKConnectVec;

BKConnectVec* bkConnectVecNew(long);
int bkConnectVecInit(BKConnectVec *,long);
int bkConnectVecEnd(BKConnectVec *);
int bkConnectVecDestroy(BKConnectVec *);
int bkConnectVecCopy(BKConnectVec *,BKConnectVec *);
int bkConnectVecPushBack(BKConnectVec *,BKConnect *);
int bkConnectVecPopBack(BKConnectVec *,BKConnect *);
int bkConnectVecResize(BKConnectVec *,long);
int bkConnectVecGet(BKConnectVec *,long,BKConnect *);
int bkConnectVecSet(BKConnectVec *,long,BKConnect *);

typedef struct BKNodeInfo_
{
    long newNeighbInd;
    double newCost;
    double cost;
    double newSum[NB_OA];
    double sum[NB_OA];
} BKNodeInfo;

typedef struct BKNode_
{
    long nodeId;
    BKConnectVec inNeighb;
    BKConnectVec outNeighb;
    long neighbInd;
    BKNodeInfo info;
} BKNode;

typedef struct BKNodeVec_
{
    long size;
    long top;
    BKNode *cont;
} BKNodeVec;

BKNodeVec* bkNodeVecNew(long);
int bkNodeVecInit(BKNodeVec *,long);
int bkNodeVecEnd(BKNodeVec *);
int bkNodeVecDestroy(BKNodeVec *);
int bkNodeVecPushBack(BKNodeVec *,BKNode *);
int bkNodeVecPopBack(BKNodeVec *,BKNode *);
int bkNodeVecResize(BKNodeVec *,long);
BKNode *bkNodeVecGet(BKNodeVec *,long);
int bkNodeVecSet(BKNodeVec *,long,BKNode *);

typedef struct BKTopology_
{
    BKNodeVec nodeVec;
    LongVec nodeInd;
    long nbNodes;
    long nbLinks;
} BKTopology;



// Auxiliary functions
//---------------------

int initTopo(BKTopology*,long);
int endTopo(BKTopology*);
int fillTopo(DataBase*,LSPRequest*,BKTopology *);
int printTopo(BKTopology*);
int getRequestSrc(LSPRequest*);
int getRequestDst(LSPRequest*);
int updateRequest(BKTopology*,LSPRequest*);
int bellmanKalaba(BKTopology*,LSPRequest*);
int initScore(long,BKTopology*);
double makeScore(BKTopology*,LSPRequest*,long,long,BKConnect*);
int updateNodeInfoOnElect(BKTopology*,LSPRequest*,long,long,BKConnect*);
int activateNodeInfo(BKTopology*,long);
int noLoop(BKTopology*,long,long);



#endif
