#ifndef DATABASE_ST_H
#define DATABASE_ST_H

#include "common/common.h"
#include "common/setup.h"
#include "error/error.h"

typedef struct DataBase_ DataBase; //defined in database_util.h

/*******************************************
 * Communication structures                *
 *******************************************/

// Label switched path representation
//------------------------------------

enum {PRIM,LOCAL_BACK,GLOBAL_BACK};
typedef unsigned char DBLSPType;

/** 
 * LSP structure.
 * Label Switched Path representation, used by #DBaddLSP.  It is often needed to translate #LSPRequest (used when computing) to #DBLabelSwitchedPath (used when adding a LSP to the database).
 */
typedef struct DBLabelSwitchedPath_
{
    long id; /**< id of the LSP */
    long noContentionId; /**< soft preemption, the id of the preempted LSP, of which resources can be used */
    int precedence; /**< preemption (or priority) level */
    double bw[NB_OA]; /**< LSP bandwidth.  Note that to be Diff-Serv Aware TE compliant, this field should be different from 0 only for one OA, because multiple OAs are not allowed on the same LSP */
    LongList forbidLinks; /**< list of the link colors that can't be used */
    LongList path; /**< LSP path */
    DBLSPType type; /**< can be PRIM, GLOBAL_BACK or LOCAL_BACK */
    long primID; /**< id of the primary LSP if the LSP is a backup */
    LongList primPath; /**< path of the primary LSP if the LSP is a backup */
    LongList backLSPIDs; /**< list of associated backup LSPs (?) */
} DBLabelSwitchedPath;


// Ordered List of LSPs
//----------------------

typedef struct DBLSPList_
{
    long size; 
    long top;
    DBLabelSwitchedPath **cont;
} DBLSPList;


/**
 * Link state structure.
 * This is the information maintained for each link.
 */

typedef struct DBLinkState_
{
    long color; /**< color */

    double cap[NB_OA]; /**< capacity per OA */

    double rbw[NB_OA][NB_PREEMPTION]; /**< bandwidth reserved by all LSPs (primary and backup).  This is not the sum of the reserved bandwidths because of backup bandwidth aggregation. */

    double pbw[NB_OA][NB_PREEMPTION]; /**< bandwidth reserved by all primary LSPs.  This is the sum of the bandwidths reserved by all primary LSPs */

    DoubleVec bbw[NB_OA][NB_PREEMPTION]; /**< bandwidth needed on this link when a failure on a certain link of the topology happens.  This is probably a "max" value. */

    DoubleVec remoteBbw[NB_OA][NB_PREEMPTION]; /**< idem as #bbw. Structured differently and not used */

    DoubleVec fbw[NB_OA][NB_PREEMPTION]; /**< bandwidth freed on this link when a failure on a certain link of the topology happens.  This is probably a "max" value. */
    DoubleVec remoteFbw[NB_OA][NB_PREEMPTION]; /**< idem as #fbw. Structured differently and not used.*/

} DBLinkState;

#endif
