#include "error.h"

#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>

/********************************************
 *   Error manager structures and handlers  *
 ********************************************/



ErrorList errorManager;


void errorInit()
{
    if ((errorManager.list=calloc(ERRORLIST_INITSIZE,sizeof(ErrorElem)))==NULL)
    {
	perror("Unable to initialize error manager");
	abort();
    }
    errorManager.top=0;
    errorManager.size=ERRORLIST_INITSIZE;
}

void addError(GravityLevel level, const char *msg, ...)
{
    va_list lst;
    void *ptr=NULL;
    char tmpmsg[ERRORMSG_SIZE];

    va_start(lst,msg);

    vsnprintf(tmpmsg,ERRORMSG_SIZE,msg,lst);
    tmpmsg[ERRORMSG_SIZE-1]='\0';


    switch (level)
    {
	case INFO:
	case WARNING:
	case CRITICAL:
	case PANIC:
	    break;
    }

    if (errorManager.top >= errorManager.size-ERROR_PROVISION) 
    {
	if (( ptr = realloc(errorManager.list, errorManager.size * 
			    2 * sizeof(ErrorElem))) == NULL) 
	{
	    if (errorManager.top < errorManager.size)
	    {
		errorManager.list[errorManager.top].gravity = CRITICAL;
		strncpy(errorManager.list[errorManager.top].message,
			"Critical lack of memory encountered while resizing error manager",
			ERRORMSG_SIZE);
		errorManager.list[errorManager.top++].message[ERRORMSG_SIZE-1]='\0'; 	
	    }
	    else
		addError(PANIC,"");
	    
	    if (errorManager.top < errorManager.size)
	    {
		errorManager.list[errorManager.top].gravity = level;
		strncpy(errorManager.list[errorManager.top].message,tmpmsg,ERRORMSG_SIZE);
		errorManager.list[errorManager.top++].message[ERRORMSG_SIZE-1]='\0'; 	
	    }
	}
	else
	{
	    errorManager.list=ptr;
	    errorManager.size*=2;
	    errorManager.list[errorManager.top].gravity = level;
	    strncpy(errorManager.list[errorManager.top].message,tmpmsg,ERRORMSG_SIZE);
	    errorManager.list[errorManager.top++].message[ERRORMSG_SIZE-1]='\0';  
	}
    }
    else
    {
	errorManager.list[errorManager.top].gravity = level;
	strncpy(errorManager.list[errorManager.top].message,tmpmsg,ERRORMSG_SIZE);
	errorManager.list[errorManager.top++].message[ERRORMSG_SIZE-1]='\0';  
    }
}

void printErrorStack() 
{
    long i;
    ErrorElem error;

    for (i=errorManager.top-1; i>=0; i--)
    {
	error = errorManager.list[i];

        switch(error.gravity)
        {
            case INFO:
                fprintf(stderr, "[INFO] ");
                break;
            case WARNING:
                fprintf(stderr, "[WARNING] ");
                break;
            case CRITICAL:
                fprintf(stderr, "[CRITICAL] ");
                break;
            case PANIC:
                fprintf(stderr, "[PANIC] ");

        }

        fprintf(stderr, "%s\n", error.message);

    }
}

void errorDestroy()
{
    free(errorManager.list);
    errorManager.top=0;
    errorManager.size=0;
}


