#include "predicate.h"
#include "computation/computation_api.h"
#include "database/database_api.h"
#include <string.h>
#include <stdio.h>

bool capacityClause(DBLinkState *ls,LSPRequest *req,double gain[NB_OA])
{
    double occupied[NB_OA],total[NB_OA];
    int i,j;
    
    for (i=0;i<NB_OA;i++)
    {
	occupied[i]=0;
	total[i]=0;
	for (j=0;j<NB_PREEMPTION;j++)
	{
	    if (j<=(damoteConfig.predicateConfig.allowReroute?req->precedence:NB_PREEMPTION))
		occupied[i]+=ls->rbw[i][j];
	    total[i]+=ls->rbw[i][j];
	}
    }
 
    for (i=0;i<NB_OA;i++)
    {
	if (ls->cap[i]<occupied[i])   
	    return FALSE;
	if (ls->cap[i]<total[i])
	    gain[i]=total[i]-ls->cap[i];
	else
	    gain[i]=0;
    }	

    return TRUE;
}

bool colorClause(DBLinkState *ls,LSPRequest *req)
{
    int i;

    for (i=0;i<req->forbidLinks.top;i++)
    {
	if (req->forbidLinks.cont[i]==ls->color)
	    return FALSE; 
    }

    return TRUE;
}

bool isValidRequestLink(DataBase *dataBase,long src,long dst,DBLinkState *ls,LSPRequest *req,double gain[NB_OA])
{
    /*if (req->id == 1781) {
	int olivier;
	fprintf(stderr,"\nbackup %ld: ", req->id);
	for (olivier=0;olivier<req->path.top;olivier++){
	    fprintf(stderr,"%ld ", req->path.cont[olivier]);
	}
	}*/

    LongList recPath;
    DBLinkState newLS;
    bool gate=TRUE,tmpgate,completePath=TRUE;
    long i;

    if (ls == NULL || req == NULL || gain==NULL) 
    { 
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return FALSE;
    }
    
    if (DBlinkStateInit(&newLS)<0)
    {
	addError(CRITICAL,"Unable to initialize link state in %s at line %d",
		 __FILE__,__LINE__);
	return FALSE;
    }

    for (i=0;i<req->path.top && completePath;i++)
    {
	if (req->path.cont[i]<0)
	{
	    completePath=FALSE;  
	}
    }
    
    if (! completePath)
    {
	if (longListInit(&recPath,-1)<0)
	{
	    addError(CRITICAL,"Unable to initialize path record structure in %s at line %d",
		     __FILE__,__LINE__);
	    return FALSE;
	}
	if (longListCopy(&recPath,&(req->path))<0)
	{
	    addError(CRITICAL,"Unable to copy path into record structure in %s at line %d",
		     __FILE__,__LINE__);
	    longListEnd(&recPath);
	    return FALSE;
	}
	req->path.top=0;
	if (longListPushBack(&(req->path),src)<0 ||
	    longListPushBack(&(req->path),dst)<0)
	{
	    addError(CRITICAL,"Unable to forge path into request in %s at line %d",
		     __FILE__,__LINE__);
	    longListEnd(&recPath);
	    return FALSE;
	}	
    }

    /*if (req->id == 1781) {
	int olivier;
	fprintf(stderr,"\nbackup %ld: ", req->id);
	for (olivier=0;olivier<req->path.top;olivier++){
	    fprintf(stderr,"%ld ", req->path.cont[olivier]);
	}
	}*/


    if (DBevalLSOnSetup(dataBase,src,dst,&newLS,ls,req)<0)
    {
	addError(CRITICAL,"Unable to update link state in %s at line %d",
		 __FILE__,__LINE__);
	if (!completePath)
	{
	    longListEnd(&recPath);
	}
	return FALSE;
    }

    if (damoteConfig.predicateConfig.capacityClause)
    {
	tmpgate=capacityClause(&newLS,req,gain);
	gate=gate && tmpgate;
    }
    
    if (damoteConfig.predicateConfig.colorClause)
    {
	tmpgate=colorClause(&newLS,req);
	gate=gate && tmpgate;
    }

    if (req->rerouteInfo.id >= 0 && req->rerouteInfo.src==src 
	&& req->rerouteInfo.dst==dst)
    {
	gate= FALSE;
    }

    if (!completePath)
    {
	if (longListCopy(&(req->path),&recPath)<0)
	{
	    addError(CRITICAL,"Unable to restore path into request in %s at line %d",
		     __FILE__,__LINE__);
	    longListEnd(&recPath);
	    return FALSE;
	}
	longListEnd(&recPath);
    }

    DBlinkStateEnd(&newLS);

    return gate;
}


bool isValidLSPLink(DataBase *dataBase,long src,long dst,DBLinkState *ls,DBLabelSwitchedPath *lsp,double gain[NB_OA])
{
    LSPRequest req;
    bool gate;

    if (ls == NULL || lsp == NULL || gain==NULL) 
    { 
	addError(CRITICAL,"Bad argument (NULL) in %s at line %d",
		 __FILE__,__LINE__);
	return FALSE;
    }
    
    if (lspRequestInit(&req)<0)
    {
	addError(CRITICAL,"Unable to initialize request in %s at line %d",
		 __FILE__,__LINE__);
	return FALSE;
    }

    req.id=lsp->id;
    req.precedence=lsp->precedence;
    req.type=lsp->type;
    req.primID=lsp->primID;
    req.rerouteInfo.id=lsp->noContentionId;
    req.rerouteInfo.src=-1;
    req.rerouteInfo.dst=-1;
    memcpy(req.bw,lsp->bw, NB_OA * sizeof(double));

    if (longListCopy(&(req.forbidLinks),&(lsp->forbidLinks))<0)
    {
	addError(CRITICAL,"Unable to initialize request in %s at line %d",
		 __FILE__,__LINE__);
	lspRequestEnd(&req);
	return FALSE;
    }

    if (longListCopy(&(req.path),&(lsp->path))<0)
    {
	addError(CRITICAL,"Unable to initialize request in %s at line %d",
		 __FILE__,__LINE__);
	lspRequestEnd(&req);
	return FALSE;
    }
    

    gate=isValidRequestLink(dataBase,src,dst,ls,&req,gain);

    lspRequestEnd(&req);

    return gate;
}


