#include "rerouting.h"
#include <stdio.h>
#include <math.h>


int chooseReroutedLSPs(int precedence,DBLinkState *state,DBLSPList *lspList,double toGain[NB_OA],
		       LongList *idList)
{
    int i,j,p;
    double need;    

    if (idList==NULL || idList->cont==NULL || state==NULL ||
	lspList==NULL || lspList->cont==NULL)
    {
	addError(CRITICAL,"Invalid arguments in %s at line %d",
		     __FILE__,__LINE__);
	return -1;
    }

    /*
    for (i=1;i<NB_OA;i++)
    {
	if (toGain[i]>DIGIT_PRECISION)
	{
	    addError(CRITICAL,"Only one ordered aggregate taken into account in %s at line %d",
		     __FILE__,__LINE__);
	    return -1;
	}
	for (j=0;j<NB_PREEMPTION;j++)
	{
	    if (state->rbw[i][j]>DIGIT_PRECISION)
	    {
		addError(CRITICAL,"Only one ordered aggregate taken into account in %s at line %d",
			 __FILE__,__LINE__);
		return -1;
	    }
	}
    }
    */

    /* find the oa in which we need bw */
    int oa=0;
    need=toGain[0];
    for (i=0; i<NB_OA; i++) {
        if (toGain[i] > DIGIT_PRECISION) {
            need = toGain[i];
            oa=i;
        }
    }

    /* build a list of LSP of current OA */
    /* This creates a new list but do not invoke the "add sorted" function as original list is already sorted*/
    /* We know the list is big enough because it has less elements than the original */
    DBLSPList* oaLspList = DBlspListNew(lspList->size);
    for (i=0;i<lspList->top;i++) {
        if (lspList->cont[i]->bw[oa] > DIGIT_PRECISION) {
            oaLspList->cont[oaLspList->top++] = lspList->cont[i];
        }
    }


    j=0;
    while (need>DIGIT_PRECISION && j<oaLspList->top)
    {
	p=oaLspList->cont[j]->precedence;
	if (p<=precedence)
	{
        DBlspListDestroy(oaLspList);
	    addError(CRITICAL,"Not enough preemptable bandwidth for LSP selection in %s at line %d",
		     __FILE__,__LINE__);
	    return -1; 
	}
	
	if (need >= state->rbw[oa][p])
	{
	    while (j<oaLspList->top && oaLspList->cont[j]->precedence==p)
	    {
		longListPushBack(idList,oaLspList->cont[j]->id);
		j++;
	    }
	    need=need-state->rbw[oa][p];
	    p=p-1;
	}
	else while (need>DIGIT_PRECISION && j<oaLspList->top)
	{
	    if (oaLspList->cont[j]->bw[oa]<=need)
	    {
		longListPushBack(idList,oaLspList->cont[j]->id);
		need=need-oaLspList->cont[j]->bw[oa];
		j++;
	    }
	    else
	    {
		while (j<oaLspList->top && oaLspList->cont[j]->bw[oa]>=need && oaLspList->cont[j]->precedence==p)
		    j++;
		longListPushBack(idList,oaLspList->cont[j-1]->id);
		need=need-oaLspList->cont[j-1]->bw[oa];
	    }
	}
    }

    if (need>DIGIT_PRECISION)
    {
    DBlspListDestroy(oaLspList);
	addError(CRITICAL,"Not enough preemptable bandwidth for LSP selection in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }
 
    DBlspListDestroy(oaLspList);
    return 0;
}


double makeRerouteScore(LSPRequest *req,double gain[NB_OA],DBLinkState *ls,int oa)
{
    int curPrec;
    double bwGained,score=0;
    
    bwGained=0;
    curPrec=NB_PREEMPTION-1;
    while (bwGained<gain[oa])
    {
	if (curPrec<=req->precedence)
	{
	    addError(CRITICAL,"internal error: impossible to realize gain in %s at line %d",
		     __FILE__,__LINE__);
	    return HUGE_VAL;
	}
	score=score+damoteConfig.reroutingConfig.scoreCoef[oa][curPrec]*
	    min(ls->rbw[oa][curPrec],gain[oa]-bwGained);	    
	bwGained=bwGained+
	    min(ls->rbw[oa][curPrec],gain[oa]-bwGained);
	curPrec--;
    }
    
    return score/req->bw[oa];
}
