#include "database/database_api.h"
#include "computation/computation_api.h"
#include "computation/backup/backup_api.h"
#include "common/common.h"
#include "error/error.h"

#include <stdio.h>
#include <string.h>

#define LINUX
#define TIME1
#define TIMING
#define SIMULATOR

#if defined LINUX && defined TIMING
#include <sys/time.h>
#endif

// Prototype for scenario :
extern int makeTopology(DataBase*);
extern int makeReqList(LSPRequestList*);

#define SOFT

int
main() 
{
   DataBase* centralDB=NULL;
   LSPRequestList reqList;
   LongVec preemptList;
   LongVec preemptList2;
   DBLabelSwitchedPath lsp;
   long i,j;
   long id=0;
   long preempt_id;
   long req_id;
   int c;
   
#if defined LINUX && defined TIMING && defined TIME1
   struct timezone tz;
   struct timeval  t1,t2;
#endif
   
   // Init the error manager ....
   errorInit();

   // Allocate a new DB
   if ((centralDB = DBnew(0)) == NULL)
     {
	addError(CRITICAL,"Impossible to init a DataBase in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
     }

   // Configuration
   damoteConfig.predicateConfig.colorClause=FALSE;
   damoteConfig.predicateConfig.allowReroute=TRUE;
   damoteConfig.predicateConfig.capacityClause=TRUE;
   
   memset(&damoteConfig.primaryComputationConfig,0,sizeof(PrimaryComputationConfig));
   damoteConfig.primaryComputationConfig.load[0]=1;
   
   memset(&damoteConfig.reroutingConfig,0,sizeof(ReroutingConfig));
   for (i=0;i<NB_OA;i++)
     for (j=0;j<NB_PREEMPTION;j++)
       damoteConfig.reroutingConfig.scoreCoef[i][j]=1;
   
   makeTopology(centralDB);
   
   // Check the allocated mem
   printf("\n-----------------------------------------------------------------------\n");
   printf("Memory allocated on the heap : %ld bytes\n", allocatedMemory);
   
   // Create the list of request
   printf("\n-----------------------------------------------------------------------\n");
   printf("Preparing request list\n");
   
   lspRequestListInit(&reqList, -1);
   makeReqList(&reqList);   
   
   // Check the allocated mem
   printf("\n-----------------------------------------------------------------------\n");
   printf("Memory allocated on the heap : %ld bytes\n", allocatedMemory);
   
   // init the lsp
   DBlspInit(&lsp);
   
   // Start establishement
   // 
#if defined LINUX && defined TIMING && defined TIME1
   gettimeofday(&t1, &tz);
#endif
   
   printf("Nb REQ = %ld\n", reqList.size);
   
   longVecInit(&preemptList,-1);
   longVecInit(&preemptList2,-1);

   for (i=0; i<reqList.size; i++) {
      
      long j,l;
      
      if (i%10 == 0){
	fprintf(stderr,"%ld already done\n", i);
	//DBprintDB(centralDB);
      }
      
      printf("BEFORE COMPUTE PRIMARY PATH\n");
      for (j=0; j<reqList.cont[i].path.top; j++) {
	 printf("%ld ", reqList.cont[i].path.cont[j]);
      }
      printf("\n");
      
     
      // compute
      if (computePrimaryPath(centralDB, &(reqList.cont[i])) >= 0) {
	 
	 // print
	 // 
	  printf("AFTER COMPUTE PRIMARY PATH\n");
	 for (j=0; j<reqList.cont[i].path.top; j++) {
	    printf("%ld ", reqList.cont[i].path.cont[j]);
	 }
	 printf("\n");
	 
	 // make the lsp
	    
	 longVecCopy(&(lsp.path), &(reqList.cont[i].path));
	 lsp.id = i;
	 lsp.bw[0] = reqList.cont[i].bw[0];
	 lsp.precedence = reqList.cont[i].precedence;
	 lsp.type = PRIM;
	 lsp.primID = -1;
	 lsp.noContentionId = -1;
	 
	 printf("Add LSP with id = %ld\n", lsp.id);
	 
	 if (DBaddLSP(centralDB, &lsp, &preemptList) < 0)
	   {
	      printf("[DBadd] Failure of LSP number %ld\n", lsp.id);
	      //olivier
	      exit(-1);
	      //id--;
	   }
	 
	 printf("------------- Nb of LSPs waiting to be rerouted : %ld\n", preemptList.top);
	 for (l=0; l<preemptList.top; l++)
	     printf("%ld-", preemptList.cont[l]);
	 printf("\n");

	 while ((longListPopBack(&preemptList, &preempt_id))!=-1) {
	    
	    printf ("here's preempt_id: %ld\n", preempt_id);

	    

#ifdef HARD
	    printf("Removing lsp number %ld\n", preempt_id);
	    if (DBremoveLSP(centralDB,preempt_id) < 0) {
		printf("[DBremove] Failure of LSP removal %ld\n", preempt_id);
		exit(-1);
	    }
#endif
	    
	    req_id = preempt_id;
	    while (req_id > 100) 
		req_id -= 100;

	    // preempt id contient de l'id du lsp qui doit tre interrompu
		
	    // on le rtablit d'abord 
	    // on ne spcifie rien dans rerouteInfo src et dest
	    // d'aprs moi c'est inutile puis toute la topologie
	    // modifie est instantanment connue
	    // par contre, il faut spcifier rerouteInfo.id !
	    // pour qu'il fasse du SE?
	    // 

#ifdef SOFT
	    reqList.cont[req_id].rerouteInfo.id = preempt_id;
	    reqList.cont[req_id].rerouteInfo.src = -1;
	    reqList.cont[req_id].rerouteInfo.dst = -1;
#endif
	    
	    printf("Coming  here, and thus rerouting!\n");
	    printf("Old route before computePrimaryPath\n");
	    
	    for (j=0; j<reqList.cont[req_id].path.top; j++) {
	       printf("%ld ", reqList.cont[req_id].path.cont[j]);
	    }
	    printf("\n");

	    reqList.cont[req_id].path.cont[2] = reqList.cont[req_id].path.cont[reqList.cont[req_id].path.top-1];
	    reqList.cont[req_id].path.cont[1] = -1;
	    reqList.cont[req_id].path.top = 3;

	    if (computePrimaryPath(centralDB, &(reqList.cont[req_id])) >= 0) {

		printf("New route after computePrimaryPath\n");
	    
		for (j=0; j<reqList.cont[req_id].path.top; j++) {
		    printf("%ld ", reqList.cont[req_id].path.cont[j]);
		}
		printf("\n");
	       
		// make the lsp
		    
		longVecCopy(&(lsp.path), &(reqList.cont[req_id].path));
		lsp.id = preempt_id+100;
		lsp.bw[0] = reqList.cont[req_id].bw[0];
		lsp.precedence = reqList.cont[req_id].precedence;
		lsp.type = PRIM;
		lsp.primID = -1;
		lsp.noContentionId = preempt_id;
	       
		printf("Add LSP with id = %ld\n", lsp.id);
		
		printf("Nb of LSPs waiting to be rerouted : %ld\n", preemptList.top);
		for (l=0; l<preemptList.top; l++)
		    printf("%ld-", preemptList.cont[l]);
		printf("\n");

		preemptList2.top = 0;
		if (DBaddLSP(centralDB, &lsp, &preemptList2) < 0)
		{
		    printf("[DBadd] Failure of LSP number %ld\n", i);
		}

		for (l=0; l<preemptList2.top; l++) {
		    printf("%ld-", preemptList2.cont[l]);
		    longListPushBack(&preemptList, preemptList2.cont[l]);
		}
	       
		printf("Nb of LSPs waiting to be rerouted : %ld\n", preemptList.top);
		for (l=0; l<preemptList.top; l++)
		    printf("%ld-", preemptList.cont[l]);
		printf("\n");
		
	    }
	    else {
		printf("Rerouting failed\n");
	    }

	    for (l=0; l<preemptList.top; l++) {
		if (preemptList.cont[l] == preempt_id) {
		    printf("+++++ Removing another occurence of preempt id ++++++\n\n");
		    int c;
		    for (c=l+1; c<preemptList.top; c++) {
			preemptList.cont[c-1] = preemptList.cont[c];
		    }
		    preemptList.top--;
		    l--;
		}
	    }

#ifdef SOFT
	    printf("Removing lsp number %ld\n", preempt_id);
	    if (DBremoveLSP(centralDB,preempt_id) < 0) {
		printf("[DBremove] Failure of LSP removal %ld\n", preempt_id);
		exit(-1);
	    }
#endif

	 }
      }
   }

   longVecEnd(&preemptList);
   longVecEnd(&preemptList2);
   
#if defined LINUX && defined TIMING && defined TIME1
   gettimeofday(&t2, &tz);
   fprintf(stderr, "Time to establish all : %f ms\n", (t2.tv_sec - t1.tv_sec) * 1000 + 
	   (t2.tv_usec - t1.tv_usec) / 1000.0);
#endif
   
    // Print the DATABASE info
   DBprintDB(centralDB);
   
   // ------------------- END TEST 1 -----------------------
    
   clean:
   
   // Destroy the request list
   lspRequestListEnd(&reqList);

   // Destroy the lsp
   DBlspEnd(&lsp);
   
   // Destroy the DB if required
   if (centralDB != NULL)
     DBdestroy(centralDB);
   
   // Print list of error
   printErrorStack();
    
   // Destroy error manager
   errorDestroy();
   
   // Check the allocated mem
   printf("\n-----------------------------------------------------------------------\n");
   printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);
    
   return 0;
}



