#include <stdio.h>
#include "database/database_api.h"
#include "computation/computation_api.h"
#include "common/common.h"
#include "error/error.h"

int
main() 
{
    DataBase* centralDB=NULL;
    DBLinkState* ls=NULL;
    long i;
    void* ptr=NULL;
    int src[] =    {0 ,1 ,0 ,1 ,1 ,2 ,2 ,3 ,10,3 ,4 ,4 ,6 ,7 ,7 ,5 ,5 ,8};
    int dst[] =    {1 ,10,2 ,2 ,3 ,3 ,5 ,5 ,4 ,4 ,6 ,7 ,7 ,5 ,8 ,8 ,9 ,9};
    double cap[] = {10,10,10,10,20,20,20,20,10,20,10,10,10,20,10,10,10,10};
    DBLabelSwitchedPath* lsp=NULL;
    LSPRequest req;

    printf("Testing realloc ...\n");

    ptr = malloc(1000);
    // Check the allocated mem
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

    ptr = realloc(ptr, 2000);
    // Check the allocated mem
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

    free(ptr);
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

    // Init the error manager ....
    errorInit();

    // Configuration
    damoteConfig.predicateConfig.colorClause=FALSE;
    damoteConfig.predicateConfig.allowReroute=FALSE;
    damoteConfig.predicateConfig.capacityClause=FALSE;

    // Allocate a new DB
    if ((centralDB = DBnew(0)) == NULL)
    {
	addError(CRITICAL,"Impossible to init a DataBase in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }
   
    // ------------------- TEST 1 -----------------------
    // Create the topology
    
    printf("Creating Nodes ...\n");
    for (i=0; i<11; i++)
    {
	if (DBaddNode(centralDB, i) != 0)
	{
	    addError(CRITICAL,"Impossible to init a Node in %s at line %d",
		     __FILE__,__LINE__);
	    goto clean;
	}
    }
   
    printf("Init of the default linkstate\n");
    if ((ls = DBlinkStateNew()) == NULL)
    {
	addError(CRITICAL,"Impossible to init a Link in %s at line %d",
		 __FILE__,__LINE__);
  	goto clean;
    }

    printf("Adding Links to the topology\n");
    for (i=0; i<18; i++)
    {
	ls->cap[0] = cap[i];
	ls->cap[1] = 0.3*cap[i];
	ls->color = 0;
	
	printf("Adding Link %ld\n", i);
	
	if (DBaddLink(centralDB, 2*i, src[i], dst[i], ls) != 0) 
	{
	    addError(CRITICAL,"Impossible to add Link in %s at line %d",
		     __FILE__,__LINE__);
	    goto clean;
	}

	if (DBaddLink(centralDB, 2*i+1, dst[i], src[i], ls) != 0) 
	{
	    addError(CRITICAL,"Impossible to add Link in %s at line %d",
		     __FILE__,__LINE__);
	    goto clean;
	}
  
    }

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory allocated on the heap : %ld bytes\n", allocatedMemory);
    
    // Clear the ls
    DBlinkStateDestroy(ls);


/*
    // new LSP
    lsp = DBlspNew();

    // primary
    lsp->id = 0;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 0;
    lsp->type = PRIM;
    lsp->primID = -1;

    longListPushBack(&(lsp->path), 0);
    longListPushBack(&(lsp->path), 2);
    longListPushBack(&(lsp->path), 3);
    longListPushBack(&(lsp->path), 5);
    longListPushBack(&(lsp->path), 7);
    longListPushBack(&(lsp->path), 6);

    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }	

    //backup
    lsp->id++;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = GLOBAL_BACK;
    lsp->primID = 0;
    longListCopy(&(lsp->primPath), &(lsp->path));
    
    lsp->path.top = 0;
    
    longListPushBack(&(lsp->path), 0);
    longListPushBack(&(lsp->path), 1);
    longListPushBack(&(lsp->path), 10);
    longListPushBack(&(lsp->path), 4);
    longListPushBack(&(lsp->path), 6);

    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    //backup l1
    lsp->id++;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = LOCAL_BACK;
    lsp->primID = 0;
    
    lsp->path.top = 0;
    
    longListPushBack(&(lsp->path), 0);
    longListPushBack(&(lsp->path), 1);
    longListPushBack(&(lsp->path), 3);
    longListPushBack(&(lsp->path), 4);
    longListPushBack(&(lsp->path), 7);

    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    //backup l2
    lsp->id++;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = LOCAL_BACK;
    lsp->primID = 0;
    
    lsp->path.top = 0;
    
    longListPushBack(&(lsp->path), 2);
    longListPushBack(&(lsp->path), 5);

    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    //backup l3
    lsp->id++;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = LOCAL_BACK;
    lsp->primID = 0;
    
    lsp->path.top = 0;
    
    longListPushBack(&(lsp->path), 3);
    longListPushBack(&(lsp->path), 4);
    longListPushBack(&(lsp->path), 7);

    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }
    
    //backup l4
    lsp->id++;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = LOCAL_BACK;
    lsp->primID = 0;
    
    lsp->path.top = 0;
    
    longListPushBack(&(lsp->path), 5);
    longListPushBack(&(lsp->path), 3);
    longListPushBack(&(lsp->path), 4);
    longListPushBack(&(lsp->path), 6);
   
    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    //backup l5
    lsp->id++;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = LOCAL_BACK;
    lsp->primID = 0;
    
    lsp->path.top = 0;
    
    longListPushBack(&(lsp->path), 7);
    longListPushBack(&(lsp->path), 4);
    longListPushBack(&(lsp->path), 6);
   
    if (DBaddLSP(centralDB, lsp) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    // remove backup
    if (DBremoveLSP(centralDB, 1) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    // Print the DATABASE info
    DBprintDB(centralDB);
    
    // ------------------- END TEST 1 -----------------------

*/

    lspRequestInit(&req);
    req.id=0;
    req.type=PRIM;
    longListPushBack(&req.path,6);
    longListPushBack(&req.path,-1);  
    longListPushBack(&req.path,0);  
    computePrimaryPath(centralDB,&req);

    printf("requested path:\n");
    lsp = DBlspNew();
    for (i=0;i<req.path.top;i++)
    {
	printf("%ld ",req.path.cont[i]);
	longListPushBack(&(lsp->path), req.path.cont[i]);
    
    }
    printf("\n\n");
    lsp->id=0;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 1;
    lsp->type = PRIM;
   
     LongVec rerouting;
     longVecInit(&rerouting,-1);

     DBaddLSP(centralDB,lsp,&rerouting);
    
    

    

    lspRequestEnd(&req);
    
 clean:
    
    // destroy LSP
/*    DBlspDestroy(lsp); */

    // Destroy the DB if required
    if (centralDB != NULL)
	DBdestroy(centralDB);
    
    // Print list of error
    printErrorStack();
    
    // Destroy error manager
    errorDestroy();
    
    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);
    
    return 0;
}
